################ Clocks ######################
## Setting up name aliases 
set int_video_clk *|ddr_sdram|*_ddr_sdram_controller_phy_inst|*_ddr_sdram_phy_inst|*_ddr_sdram_phy_alt_mem_phy_inst|clk|pll|altpll_component|auto_generated|pll1|clk[1]

## Derive the video clock offchip
create_generated_clock -name video_clk -source [get_pins $int_video_clk] [get_ports video_clk]

################ Flash ######################
# data/address bus shared via Tristate bridge
set_min_delay 0.0 -from [get_ports {flash_d*}] -to *
set_max_delay 8.0 -from [get_ports {flash_d*}] -to *

set_min_delay  0.0 -from * -to [get_ports {flash_a* flash_d* flash_chipselect_n flash_read_n flash_write_n flash_reset_n}]
set_max_delay  8.0 -from * -to [get_ports {flash_a* flash_d* flash_chipselect_n flash_read_n flash_write_n flash_reset_n}]

################ EEPROM ######################
set_min_delay 0.0 -from [get_ports lcd_id_i2cdat] -to *
set_max_delay 8.0 -from [get_ports lcd_id_i2cdat] -to *

set_min_delay  0.0 -from * -to [get_ports {lcd_id_i2cdat lcd_id_i2cscl}]
set_max_delay  8.0 -from * -to [get_ports {lcd_id_i2cdat lcd_id_i2cscl}]

################# LCD ###########################
## Constraining the LCD interface (worst case Tsu = 1.549ns, Th = 0.298ns for the offchip video interface).
## Using overestimates since board trace lengths haven't been factored in and to give some timing margin.
set_output_delay -clock video_clk -max 2 [get_ports lcd_den]
set_output_delay -clock video_clk -min -1 [get_ports lcd_den]
set_output_delay -clock video_clk -max 2 [get_ports lcd_hd]
set_output_delay -clock video_clk -min -1 [get_ports lcd_hd]
set_output_delay -clock video_clk -max 2 [get_ports lcd_data[*]]
set_output_delay -clock video_clk -min -1 [get_ports lcd_data[*]]
set_output_delay -clock video_clk -max 2 [get_ports lcd_vd]
set_output_delay -clock video_clk -min -1 [get_ports lcd_vd]
set_output_delay -clock video_clk 0.5 [get_ports video_clk]

## Cutting paths for the slow I/O
set_false_path -to [get_ports {lcd_adc_*}]
set_false_path -from [get_ports {lcd_adc_*}]
set_false_path -to [get_ports {lcd_id_*}]
set_false_path -to [get_ports {lcd_scen}]
set_false_path -to [get_ports {lcd_sda}]
set_false_path -from [get_ports {lcd_sda}]

#################### JTAG ################################
# JTAG Constraints copied directly from TimeQuest Cookbook
#constrain the TCK port
create_clock -name tck -period "10MHz" [get_ports altera_reserved_tck] -add
#cut all paths to and from tck
set_clock_groups -group [get_clocks tck] -exclusive
#constrain the TDI port
set_input_delay -clock tck 20 [get_ports altera_reserved_tdi]
#constrain the TMS port
set_input_delay -clock tck 20 [get_ports altera_reserved_tms]
#constrain the TDO port
set_output_delay -clock tck 20 [get_ports altera_reserved_tdo]

